PROGRAM_NAME='RADIONICS, D9133 SDI INTERFACE, RS-232, 3-00, RDW'
(*   DATE:10/03/00    TIME:16:58:59    *)
(***********************************************************)
(* The following code block is provided as a guide to      *)
(* programming the device(s) listed above. This is a       *)
(* sample only, and will most likely require modification  *)
(* to be integrated into a master program.                 *)
(*                                                         *)
(* Device-specific protocols should be obtained directly   *)
(* from the equipment manufacturer to ensure compliance    *)
(* with the most current versions. Within the limits       *)
(* imposed by any non-disclosure agreements, AMX will      *)
(* provide protocols upon request.                         *)
(*                                                         *)
(***********************************************************)
(* STATUS: THIS BLOCK IS BETA SINCE THE FIRMWARE ON THE    *)
(*         UNIT CHANGED AND WE HAVE NOT TESTED WITH THE    *)
(*         NEW FIRMWARE.  ONLY STATUSAND BASIC COMMANDS    *)
(*         HAVE BEEN IMPLEMENTED                           *)
(***********************************************************)
(*          DEVICE NUMBER DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_DEVICE

SECURITY1     = 96                 (* AXB-232: RADIONICS SECURITY SYSTEM 9600 8N1 *)
TP            = 128

(***********************************************************)
(*               CONSTANT DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_CONSTANT

(* SECURITY STUFF - STATUS INTERFACE *)
(* COMMANDS *)
CMD_STATUS_UPDATE         = $01
CMD_ACT_PT_STATUS         = $02
CMD_ALARM_PT_STATUS       = $03
CMD_EXT_RELAY_STATUS      = $04
CMD_SUPERVISED_PT_STATUS  = $05
CMD_FORCE_PT_STATUS       = $06
CMD_TROUBLED_PT_STATUS    = $07
CMD_BYPASSED_PT_STATUS    = $08
CMD_PANEL_STATUS          = $09
CMD_AREA_STATUS           = $0A
CMD_DOOR_STATUS           = $0C
CMD_PT_TEXT               = $0D
CMD_VERSION               = $0E
CMD_EVENT_LOG             = $0F
CMD_PT_CONDITION          = $10
CMD_INT_RELAY_STATUS      = $11
CMD_INFO_REQUEST          = $75
CMD_CHANGE_PNL_STATUS     = $76
CMD_CHANGE_AREA_STATUS    = $77
INFO_VECTOR               = $01
INFO_VERSION              = $02
VCT_ALARM_PT              = $01
VCT_EXT_RELAY             = $02
VCT_SUPERVISED_PT_ACK     = $03
VCR_FORCED_PT             = $04
VCT_BYPASSED_PT           = $05
VCT_PANEL_STATUS          = $06
VCT_AREA_STATUS           = $07
VCT_TROUBLED_PT           = $08
VCT_PT_CONDITION          = $09
VCT_INT_RELAY             = $0A
VCT_DOOR_STATUS           = $0B

(* AREA STATUS *)
AS_ARMED                  = $01
AS_PERIMETER_INSTANT_ARM  = $02
AS_PERIMETER_DELAY_ARM    = $03
AS_DISARMED               = $04
AS_AREA_ENTERY_DELAY      = $05
AS_PERIMETER_ENTRY_DELAY  = $06
AS_AREA_EXIT_DELAY        = $07
AS_PERIMETER_EXIT_DELAY   = $08
AS_MASTER_INSTANT_ARM     = $09

(* PANEL STATUS *)
PS_LOG_LIMIT_REACHED      = $8000
PS_LOG_WRAPPED            = $4000
PS_PT_BUS_FALILED         = $2000
PS_VALID_LOCAL_ACCESS     = $1000
PS_RF_TROUBLE             = $0800
PS_FAIL_TO_CALL_RAM       = $0400
PS_USER_CODE_TAMPER       = $0200
PS_SDI_DEVICE_FAILED      = $0100
PS_RECEIVER_COMM_FAILED   = $0080
PS_AC_FAILED              = $0020
PS_BETTERY_MISSING        = $0010
PS_BATTERY_LOW            = $0008
PS_CHECKSUM_FAIL          = $0004
PS_PHONE_FAIL             = $0002
PS_EXTRA_RF_PT            = $0001

(* CHANGE AREA STATUS *)
CAS_DISARM               = $01
CAS_ARM_DELAY            = $02
CAS_ARM_INSTANT          = $03
CAS_PERIM_DELAY          = $04
CAS_PERIM_INSTANT        = $05

(***********************************************************)
(*               VARIABLE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_VARIABLE

(* SECURITY - STATUS INTERFACE *)
(* ALL 32 BYTE STATUS STRINGS HOLD BIT-PACKED VERSION OF STATUS *)
(* FOR UP TO 248 POINTS.  0 IS NORMAL, 1 IS NOT NORMAL          *)
(* BIT VALUES ARE ASS FOLLOWS:  *)
(* BYTE 1, BIT 7 = POINT 1 *)
(* BYTE 1, BIT 6 = POINT 2 *)
(* BYTE 1, BIT 5 = POINT 3 *)
(* BYTE 1, BIT 4 = POINT 4 *)
(* BYTE 1, BIT 3 = POINT 5 *)
(* BYTE 1, BIT 2 = POINT 6 *)
(* BYTE 1, BIT 1 = POINT 7 *)
(* BYTE 1, BIT 0 = POINT 8 *)
(* BYTE 2, BIT 7 = POINT 9 *)
(* ... *)
(* BYTE 31, BIT 0 = POINT 248 *)
DEBUG_SEC_STATUS                   (* 1 TO DEBUG *)
SEC_POLL_INDEX                     (* POLLING FOR... *)
SEC_BUFFER[255]                    (* INCOMING BUFFER *)
SEC_TRASH[255]                     (* TRASH *)
SEC_BYTE                           (* FOR PARSING ABOVE *)
SEC_TRASH_INT                      (* "" *)
SEC_MESS_ID                        (* MESSAGE ID *)
SEC_PT_STATUS[32]                  (* POINT STATUS *)
SEC_AREA_STATUS[8]                 (* AREA STATUS *)
SEC_PANEL_STATUS[2]                (* PANEL STATUS *)
SEC_EXT_RELAY_STATUS[16]           (* STATUS OF EXTERNAL RELAYS *)
SEC_INT_RELAY_STATUS               (* STATUS OF INTERAL RELAYS *)
SEC_PT_ACK_STATUS[32]              (* POINT WENT NORMAL NOT ACKNOWLEDGED *)
SEC_SUP_PT_ACK_STATUS[32]          (* SUPERVISED POINT WENT NORMAL NOT ACKNOWLEDGED *)
SEC_FORCE_PT_STATUS[32]            (* POINT WAS FORCED *)
SEC_TROUBLED_PT_STATUS[32]         (* TROUBLED POINT STATUS *)
SEC_BYPASSED_PT_STATUS[32]         (* BYPASSED POINT STATUS *)
SEC_DOOR_STATUS[8]                 (* DOOR STATUS *)
SEC_PT_NUM                         (* POINT NUMBER OF PARSING TEXT *)
SEC_PT_TEXT[255][16]               (* POINT TEXT *)
SEC_VERSION[10]                    (* VERSION OF PANEL *)
SEC_VER1                           (* FOR BUILDING VERSION *)
SEC_VER2                           (* FOR BUILDING VERSION *)
SEC_PT_CONDITION[62]               (* CONDITION OF POINTS *)
SEC_NACK_INFO                      (* NACK INFO *)

(* GENERAL *)
ZONE                               (* ZONE WE SELECTED *)

(***********************************************************)
(*               LATCHING DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_LATCHING

(***********************************************************)
(*       MUTUALLY EXCLUSIVE DEFINITIONS GO BELOW           *)
(***********************************************************)
DEFINE_MUTUALLY_EXCLUSIVE

(***********************************************************)
(*           SUBROUTINE DEFINITIONS GO BELOW               *)
(***********************************************************)

(**************************************)
(* CALL NAME: SEND TO SEC             *)
(* FUNCTION:  SEND A MESSAGE TO ALARM *)
(*            STATUS INTERFACE        *)
(**************************************)
DEFINE_CALL 'SEND TO SEC' (CARD,CMD,DATA[30])
LOCAL_VAR LOOP S2S[30] CKSUM
{
  S2S = "CMD,LENGTH_STRING(DATA)+1"
  CKSUM = CMD+LENGTH_STRING(DATA)+1
  LOOP = 1
  WHILE (LOOP <= LENGTH_STRING(DATA))
  {
    CKSUM = (CKSUM + DATA[LOOP]) & $FF
    LOOP = LOOP + 1
  }
  CKSUM = 255 - CKSUM
  S2S = "S2S,DATA,CKSUM"
  SEND_STRING CARD,"S2S"
}

(***********************************************************)
(*                STARTUP CODE GOES BELOW                  *)
(***********************************************************)
DEFINE_START

(* BUFFERS *)
CREATE_BUFFER SECURITY1,SEC_BUFFER (* STATUS INTERFACE *)
OFF[DEBUG_SEC_STATUS]

(***********************************************************)
(*            THE ACTUAL PROGRAM GOES BELOW                *)
(***********************************************************)
DEFINE_PROGRAM

(* CONTROL *************************************************)
PUSH[TP,1]                         (* DISARM *)
{
  ZONE = 1 (* 0-8 OR 0 FOR ALL *)
  CALL 'SEND TO SEC' (SECURITY1,CMD_CHANGE_AREA_STATUS,"CAS_DISARM,ZONE")
}
[TP,1] = (SEC_AREA_STATUS[1] = AS_DISARMED)

PUSH[TP,2]                         (* ARM AWAY DELAY *)
{
  ZONE = 1 (* 0-8 OR 0 FOR ALL *)
  CALL 'SEND TO SEC' (SECURITY1,CMD_CHANGE_AREA_STATUS,"CAS_ARM_DELAY,ZONE")
}
[TP,2] = (SEC_AREA_STATUS[1] = AS_ARMED || SEC_AREA_STATUS[1] = AS_AREA_EXIT_DELAY)

PUSH[TP,3]                         (* ARM AWAY INSTANT *)
{
  ZONE = 1 (* 0-8 OR 0 FOR ALL *)
  CALL 'SEND TO SEC' (SECURITY1,CMD_CHANGE_AREA_STATUS,"CAS_ARM_INSTANT,ZONE")
}
[TP,3] = (SEC_AREA_STATUS[1] = AS_ARMED)

PUSH[TP,4]                         (* ARM HOME DELAY *)
{
  ZONE = 1 (* 0-8 OR 0 FOR ALL *)
  CALL 'SEND TO SEC' (SECURITY1,CMD_CHANGE_AREA_STATUS,"CAS_PERIM_DELAY,ZONE")
}
[TP,4] = (SEC_AREA_STATUS[1] = AS_PERIMETER_DELAY_ARM ||
          SEC_AREA_STATUS[1] = AS_PERIMETER_EXIT_DELAY)

PUSH[TP,5]                         (* ARM HOME INSTANT *)
{
  ZONE = 1 (* 0-8 OR 0 FOR ALL *)
  CALL 'SEND TO SEC' (SECURITY1,CMD_CHANGE_AREA_STATUS,"CAS_PERIM_INSTANT,ZONE")
}
[TP,5] = (SEC_AREA_STATUS[1] = AS_PERIMETER_INSTANT_ARM)

(* POLLING *************************************************)
WAIT 30
{
  SEC_POLL_INDEX = SEC_POLL_INDEX + 1
  IF (SEC_POLL_INDEX = 0) SEC_POLL_INDEX = 1
  IF (SEC_POLL_INDEX >= 12) SEC_POLL_INDEX = 1
  IF (SEC_POLL_INDEX = 1)
    CALL 'SEND TO SEC' (SECURITY1,CMD_INFO_REQUEST,"INFO_VERSION")
  ELSE
    CALL 'SEND TO SEC' (SECURITY1,CMD_INFO_REQUEST,"INFO_VECTOR,SEC_POLL_INDEX-1")
}

(* BUFFER PROCESSING ***************************************)
IF (LENGTH_STRING(SEC_BUFFER))
{
  SEC_BYTE = GET_BUFFER_CHAR(SEC_BUFFER)
  SELECT
  {
    (* DEFUALT PANEL STATUS *)
    ACTIVE (SEC_BYTE = CMD_STATUS_UPDATE && SEC_BUFFER[1] = $3C &&
            LENGTH_STRING(SEC_BUFFER) >= 58):
    {
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      SEC_MESS_ID   = GET_BUFFER_CHAR(SEC_BUFFER)
      SEC_PT_STATUS = LEFT_STRING(SEC_BUFFER,31)
      SEC_TRASH = REMOVE_STRING(SEC_BUFFER,SEC_PT_STATUS,1)
      SEC_AREA_STATUS = LEFT_STRING(SEC_BUFFER,8)
      SEC_TRASH = REMOVE_STRING(SEC_BUFFER,SEC_AREA_STATUS,1)
      SEC_PANEL_STATUS = LEFT_STRING(SEC_BUFFER,2)
      SEC_TRASH = REMOVE_STRING(SEC_BUFFER,SEC_PANEL_STATUS,1)
      SEC_EXT_RELAY_STATUS = LEFT_STRING(SEC_BUFFER,16)
      SEC_TRASH = REMOVE_STRING(SEC_BUFFER,SEC_EXT_RELAY_STATUS,1)
      SEC_INT_RELAY_STATUS = GET_BUFFER_CHAR(SEC_BUFFER)
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      IF (DEBUG_SEC_STATUS)
        SEND_STRING 0,"'SEC: GOT DEFUALT STATUS',13,10"
    }
    ACTIVE (SEC_BYTE = CMD_STATUS_UPDATE && SEC_BUFFER[1] = $3C):
      SEC_BUFFER = "SEC_BYTE,SEC_BUFFER"
    (* DEFUALT PANEL STATUS *)
    ACTIVE (SEC_BYTE = CMD_STATUS_UPDATE && SEC_BUFFER[1] = $2B &&
            LENGTH_STRING(SEC_BUFFER) >= 44):
    {
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      SEC_MESS_ID   = GET_BUFFER_CHAR(SEC_BUFFER)
      SEC_PT_STATUS = LEFT_STRING(SEC_BUFFER,31)
      SEC_TRASH = REMOVE_STRING(SEC_BUFFER,SEC_PT_STATUS,1)
      SEC_AREA_STATUS = LEFT_STRING(SEC_BUFFER,8)
      SEC_TRASH = REMOVE_STRING(SEC_BUFFER,SEC_AREA_STATUS,1)
      SEC_PANEL_STATUS = LEFT_STRING(SEC_BUFFER,2)
      SEC_TRASH = REMOVE_STRING(SEC_BUFFER,SEC_PANEL_STATUS,1)
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      IF (DEBUG_SEC_STATUS)
        SEND_STRING 0,"'SEC: GOT DEFUALT STATUS',13,10"
    }
    ACTIVE (SEC_BYTE = CMD_STATUS_UPDATE && SEC_BUFFER[1] = $2B):
      SEC_BUFFER = "SEC_BYTE,SEC_BUFFER"
    (* ACTUAL POINT STATUS *)
    ACTIVE (SEC_BYTE = CMD_ACT_PT_STATUS && SEC_BUFFER[1] = $20 &&
            LENGTH_STRING(SEC_BUFFER) >= 33):
    {
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      SEC_PT_STATUS = LEFT_STRING(SEC_BUFFER,31)
      SEC_TRASH = REMOVE_STRING(SEC_BUFFER,SEC_PT_STATUS,1)
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      IF (DEBUG_SEC_STATUS)
        SEND_STRING 0,"'SEC: GOT POINT STATUS',13,10"
    }
    ACTIVE (SEC_BYTE = CMD_ACT_PT_STATUS && SEC_BUFFER[1] = $20):
      SEC_BUFFER = "SEC_BYTE,SEC_BUFFER"
    (* UNACKED ALARM POINT STATUS *)
    ACTIVE (SEC_BYTE = CMD_ALARM_PT_STATUS && SEC_BUFFER[1] = $20 &&
            LENGTH_STRING(SEC_BUFFER) >= 33):
    {
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      SEC_PT_ACK_STATUS = LEFT_STRING(SEC_BUFFER,31)
      SEC_TRASH = REMOVE_STRING(SEC_BUFFER,SEC_PT_ACK_STATUS,1)
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      IF (DEBUG_SEC_STATUS)
        SEND_STRING 0,"'SEC: GOT ALARM POINT STATUS',13,10"
    }
    ACTIVE (SEC_BYTE = CMD_ALARM_PT_STATUS && SEC_BUFFER[1] = $20):
      SEC_BUFFER = "SEC_BYTE,SEC_BUFFER"
    (* EXT RELAY STATUS *)
    ACTIVE (SEC_BYTE = CMD_EXT_RELAY_STATUS && SEC_BUFFER[1] = $11 &&
            LENGTH_STRING(SEC_BUFFER) >= 18):
    {
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      SEC_EXT_RELAY_STATUS = LEFT_STRING(SEC_BUFFER,16)
      SEC_TRASH = REMOVE_STRING(SEC_BUFFER,SEC_EXT_RELAY_STATUS,1)
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      IF (DEBUG_SEC_STATUS)
        SEND_STRING 0,"'SEC: GOT EXTERNAL RELAY STATUS',13,10"
    }
    ACTIVE (SEC_BYTE = CMD_EXT_RELAY_STATUS && SEC_BUFFER[1] = $11):
      SEC_BUFFER = "SEC_BYTE,SEC_BUFFER"
    (* UNACKED SUPERVISED POINT STATUS *)
    ACTIVE (SEC_BYTE = CMD_SUPERVISED_PT_STATUS && SEC_BUFFER[1] = $20 &&
            LENGTH_STRING(SEC_BUFFER) >= 33):
    {
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      SEC_SUP_PT_ACK_STATUS = LEFT_STRING(SEC_BUFFER,31)
      SEC_TRASH = REMOVE_STRING(SEC_BUFFER,SEC_SUP_PT_ACK_STATUS,1)
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      IF (DEBUG_SEC_STATUS)
        SEND_STRING 0,"'SEC: GOT SUPERVISED POINT STATUS',13,10"
    }
    ACTIVE (SEC_BYTE = CMD_SUPERVISED_PT_STATUS && SEC_BUFFER[1] = $20):
      SEC_BUFFER = "SEC_BYTE,SEC_BUFFER"
    (* FORCED POINT STATUS *)
    ACTIVE (SEC_BYTE = CMD_FORCE_PT_STATUS && SEC_BUFFER[1] = $20 &&
            LENGTH_STRING(SEC_BUFFER) >= 33):
    {
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      SEC_FORCE_PT_STATUS = LEFT_STRING(SEC_BUFFER,31)
      SEC_TRASH = REMOVE_STRING(SEC_BUFFER,SEC_FORCE_PT_STATUS,1)
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      IF (DEBUG_SEC_STATUS)
        SEND_STRING 0,"'SEC: GOT FORCE POINT STATUS',13,10"
    }
    ACTIVE (SEC_BYTE = CMD_FORCE_PT_STATUS && SEC_BUFFER[1] = $20):
      SEC_BUFFER = "SEC_BYTE,SEC_BUFFER"
    (* TROUBLED POINT STATUS *)
    ACTIVE (SEC_BYTE = CMD_TROUBLED_PT_STATUS && SEC_BUFFER[1] = $20 &&
            LENGTH_STRING(SEC_BUFFER) >= 33):
    {
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      SEC_TROUBLED_PT_STATUS = LEFT_STRING(SEC_BUFFER,31)
      SEC_TRASH = REMOVE_STRING(SEC_BUFFER,SEC_TROUBLED_PT_STATUS,1)
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      IF (DEBUG_SEC_STATUS)
        SEND_STRING 0,"'SEC: GOT TROUBLED POINT STATUS',13,10"
    }
    ACTIVE (SEC_BYTE = CMD_TROUBLED_PT_STATUS && SEC_BUFFER[1] = $20):
      SEC_BUFFER = "SEC_BYTE,SEC_BUFFER"
    (* BYPASSED POINT STATUS *)
    ACTIVE (SEC_BYTE = CMD_BYPASSED_PT_STATUS && SEC_BUFFER[1] = $20 &&
            LENGTH_STRING(SEC_BUFFER) >= 33):
    {
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      SEC_BYPASSED_PT_STATUS = LEFT_STRING(SEC_BUFFER,31)
      SEC_TRASH = REMOVE_STRING(SEC_BUFFER,SEC_BYPASSED_PT_STATUS,1)
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      IF (DEBUG_SEC_STATUS)
        SEND_STRING 0,"'SEC: GOT BYPASSED POINT STATUS',13,10"
    }
    ACTIVE (SEC_BYTE = CMD_BYPASSED_PT_STATUS && SEC_BUFFER[1] = $20):
      SEC_BUFFER = "SEC_BYTE,SEC_BUFFER"
    (* PANEL STATUS *)
    ACTIVE (SEC_BYTE = CMD_PANEL_STATUS && SEC_BUFFER[1] = $03 &&
            LENGTH_STRING(SEC_BUFFER) >= 4):
    {
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      SEC_PANEL_STATUS = LEFT_STRING(SEC_BUFFER,2)
      SEC_TRASH = REMOVE_STRING(SEC_BUFFER,SEC_PANEL_STATUS,1)
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      IF (DEBUG_SEC_STATUS)
        SEND_STRING 0,"'SEC: GOT PANEL STATUS',13,10"
    }
    ACTIVE (SEC_BYTE = CMD_PANEL_STATUS && SEC_BUFFER[1] = $03):
      SEC_BUFFER = "SEC_BYTE,SEC_BUFFER"
    (* PANEL STATUS *)
    ACTIVE (SEC_BYTE = CMD_AREA_STATUS && SEC_BUFFER[1] = $09 &&
            LENGTH_STRING(SEC_BUFFER) >= 9):
    {
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      SEC_AREA_STATUS = LEFT_STRING(SEC_BUFFER,8)
      SEC_TRASH = REMOVE_STRING(SEC_BUFFER,SEC_AREA_STATUS,1)
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      IF (DEBUG_SEC_STATUS)
        SEND_STRING 0,"'SEC: GOT AREA STATUS',13,10"
    }
    ACTIVE (SEC_BYTE = CMD_AREA_STATUS && SEC_BUFFER[1] = $09):
      SEC_BUFFER = "SEC_BYTE,SEC_BUFFER"
    (* DOOR STATUS *)
    ACTIVE (SEC_BYTE = CMD_DOOR_STATUS && SEC_BUFFER[1] = $09 &&
            LENGTH_STRING(SEC_BUFFER) >= 3):
    {
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      SEC_DOOR_STATUS = LEFT_STRING(SEC_BUFFER,8)
      SEC_TRASH = REMOVE_STRING(SEC_BUFFER,SEC_DOOR_STATUS,1)
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      IF (DEBUG_SEC_STATUS)
        SEND_STRING 0,"'SEC: GOT DOOR STATUS',13,10"
    }
    ACTIVE (SEC_BYTE = CMD_DOOR_STATUS && SEC_BUFFER[1] = $09):
      SEC_BUFFER = "SEC_BYTE,SEC_BUFFER"
    (* POINT TEXT *)
    ACTIVE (SEC_BYTE = CMD_PT_TEXT && SEC_BUFFER[1] = $12 &&
            LENGTH_STRING(SEC_BUFFER) >= 19):
    {
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      SEC_PT_NUM = GET_BUFFER_CHAR(SEC_BUFFER)
      (* IF POINT IS VALID *)
      IF (SEC_PT_NUM)
      {
        SEC_PT_TEXT[SEC_PT_NUM] = LEFT_STRING(SEC_BUFFER,16)
        SEC_TRASH = REMOVE_STRING(SEC_BUFFER,SEC_PT_TEXT[SEC_PT_NUM],1)
        SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
        IF (DEBUG_SEC_STATUS)
          SEND_STRING 0,"'SEC: GOT POINT TEXT: ',SEC_PT_TEXT[SEC_PT_NUM],13,10"
      }
      (* ELSE CLEAN UP BUFFER *)
      ELSE
      {
        SEC_TRASH = LEFT_STRING(SEC_BUFFER,18)
        SEC_TRASH = REMOVE_STRING(SEC_BUFFER,SEC_TRASH,1)
        IF (DEBUG_SEC_STATUS)
          SEND_STRING 0,"'SEC: GOT POINT TEXT FOR BAD POINT NUM',13,10"
      }
    }
    ACTIVE (SEC_BYTE = CMD_PT_TEXT && SEC_BUFFER[1] = $12):
      SEC_BUFFER = "SEC_BYTE,SEC_BUFFER"
    (* VERSION *)
    ACTIVE (SEC_BYTE = CMD_VERSION && SEC_BUFFER[1] = $03 &&
            LENGTH_STRING(SEC_BUFFER) >= 4):
    {
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      SEC_VER1 = GET_BUFFER_CHAR(SEC_BUFFER)
      SEC_VER2 = GET_BUFFER_CHAR(SEC_BUFFER)
      SEC_VERSION = "ITOA(SEC_VER1),'.',ITOA(SEC_VER2)"
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      IF (DEBUG_SEC_STATUS)
        SEND_STRING 0,"'SEC: GOT VERSION: ',SEC_VERSION,13,10"
    }
    ACTIVE (SEC_BYTE = CMD_VERSION && SEC_BUFFER[1] = $03):
      SEC_BUFFER = "SEC_BYTE,SEC_BUFFER"
    (* POINT CONDITION *)
    ACTIVE (SEC_BYTE = CMD_PT_CONDITION && SEC_BUFFER[1] = $3F &&
            LENGTH_STRING(SEC_BUFFER) >= 64):
    {
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      SEC_PT_CONDITION = LEFT_STRING(SEC_BUFFER,62)
      SEC_TRASH = REMOVE_STRING(SEC_BUFFER,SEC_PT_CONDITION,1)
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      IF (DEBUG_SEC_STATUS)
        SEND_STRING 0,"'SEC: GOT POINT CONDITION STATUS',13,10"
    }
    ACTIVE (SEC_BYTE = CMD_PT_CONDITION && SEC_BUFFER[1] = $3F):
      SEC_BUFFER = "SEC_BYTE,SEC_BUFFER"
    (* INT RELAY STATUS *)
    ACTIVE (SEC_BYTE = CMD_INT_RELAY_STATUS && SEC_BUFFER[1] = $02 &&
            LENGTH_STRING(SEC_BUFFER) >= 3):
    {
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      SEC_INT_RELAY_STATUS = GET_BUFFER_CHAR(SEC_BUFFER)
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      IF (DEBUG_SEC_STATUS)
        SEND_STRING 0,"'SEC: GOT INTERNAL RELAY STATUS',13,10"
    }
    ACTIVE (SEC_BYTE = CMD_INT_RELAY_STATUS && SEC_BUFFER[1] = $02):
      SEC_BUFFER = "SEC_BYTE,SEC_BUFFER"
    (* ACK *)
    ACTIVE (SEC_BYTE = $FA && SEC_BUFFER[1] = $02 &&
            LENGTH_STRING(SEC_BUFFER) >= 3):
    {
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      IF (DEBUG_SEC_STATUS)
        SEND_STRING 0,"'SEC: GOT ACK',13,10"
    }
    ACTIVE (SEC_BYTE = $FA && SEC_BUFFER[1] = $02):
      SEC_BUFFER = "SEC_BYTE,SEC_BUFFER"
    (* NACK *)
    ACTIVE (SEC_BYTE = $FC && SEC_BUFFER[1] = $02 &&
            LENGTH_STRING(SEC_BUFFER) >= 3):
    {
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      SEC_NACK_INFO = GET_BUFFER_CHAR(SEC_BUFFER)
      SEC_TRASH_INT = GET_BUFFER_CHAR(SEC_BUFFER)
      IF (DEBUG_SEC_STATUS)
      {
        SEND_STRING 0,"'SEC: GOT NACK - '"
        SELECT
        {
          ACTIVE (SEC_NACK_INFO = 0):
            SEND_STRING 0,"'UNKNOWN',13,10"
          ACTIVE (SEC_NACK_INFO = 1):
            SEND_STRING 0,"'BAD CHECKSUM',13,10"
          ACTIVE (SEC_NACK_INFO = 2):
            SEND_STRING 0,"'BAD COMMAND',13,10"
          ACTIVE (SEC_NACK_INFO = 3):
            SEND_STRING 0,"'BAD FORMAT',13,10"
          ACTIVE (SEC_NACK_INFO = 4):
            SEND_STRING 0,"'BAD SYNTAX',13,10"
          ACTIVE (SEC_NACK_INFO = 5):
            SEND_STRING 0,"'BAD REQUEST',13,10"
          ACTIVE (1):
            SEND_STRING 0,"'RESERVED????????',13,10"
        }
      }
    }
    ACTIVE (SEC_BYTE = $FC && SEC_BUFFER[1] = $02):
      SEC_BUFFER = "SEC_BYTE,SEC_BUFFER"
    (* SOME SORT OF ECHO *)
    ACTIVE (SEC_BYTE = $11):
    { (* JUST REMOVE IT AND SHUT UP! *)
    }
    (* UNKNOWN *)
    ACTIVE (1):
    {
      IF (DEBUG_SEC_STATUS)
        SEND_STRING 0,"'SEC: GOT BAD CMD: ',ITOHEX(SEC_BYTE),13,10"
    }
  }
}

(***********************************************************)
(*                     END OF PROGRAM                      *)
(*        DO NOT PUT ANY CODE BELOW THIS COMMENT           *)
(***********************************************************)
